@@ -1,5 +1,5 @@
 /*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2015 Antonio Diaz Diaz.
+    Copyright (C) 2010-2016 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #define _FILE_OFFSET_BITS 64
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
@@ -66,10 +67,11 @@
 #error "Environments where CHAR_BIT != 8 are not supported."
 #endif
 
+int verbosity = 0;
 
 const char * const Program_name = "Clzip";
 const char * const program_name = "clzip";
-const char * const program_year = "2015";
+const char * const program_year = "2016";
 const char * invocation_name = 0;
 
 struct { const char * from; const char * to; } const known_extensions[] = {
@@ -87,10 +89,6 @@ enum Mode { m_compress, m_decompress, m_test };
 
 char * output_filename = 0;
 int outfd = -1;
-int verbosity = 0;
-const mode_t usr_rw = S_IRUSR | S_IWUSR;
-const mode_t all_rw = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
-mode_t outfd_mode = S_IRUSR | S_IWUSR;
 bool delete_output_on_interrupt = false;
 
 
@@ -101,14 +99,15 @@ static void show_help( void )
   printf( "\nOptions:\n"
           "  -h, --help                     display this help and exit\n"
           "  -V, --version                  output version information and exit\n"
+          "  -a, --trailing-error           exit with error status if trailing data\n"
           "  -b, --member-size=<bytes>      set member size limit in bytes\n"
-          "  -c, --stdout                   send output to standard output\n"
+          "  -c, --stdout                   write to standard output, keep input files\n"
           "  -d, --decompress               decompress\n"
           "  -f, --force                    overwrite existing output files\n"
           "  -F, --recompress               force re-compression of compressed files\n"
           "  -k, --keep                     keep (don't delete) input files\n"
           "  -m, --match-length=<bytes>     set match length limit in bytes [36]\n"
-          "  -o, --output=<file>            if reading stdin, place the output into <file>\n"
+          "  -o, --output=<file>            if reading standard input, write to <file>\n"
           "  -q, --quiet                    suppress all messages\n"
           "  -s, --dictionary-size=<bytes>  set dictionary size limit in bytes [8 MiB]\n"
           "  -S, --volume-size=<bytes>      set volume size limit in bytes\n"
@@ -117,13 +116,15 @@ static void show_help( void )
           "  -0 .. -9                       set compression level [default 6]\n"
           "      --fast                     alias for -0\n"
           "      --best                     alias for -9\n"
-          "If no file names are given, clzip compresses or decompresses\n"
-          "from standard input to standard output.\n"
+          "If no file names are given, or if a file is '-', clzip compresses or\n"
+          "decompresses from standard input to standard output.\n"
           "Numbers may be followed by a multiplier: k = kB = 10^3 = 1000,\n"
           "Ki = KiB = 2^10 = 1024, M = 10^6, Mi = 2^20, G = 10^9, Gi = 2^30, etc...\n"
-          "The bidimensional parameter space of LZMA can't be mapped to a linear\n"
+          "Dictionary sizes 12 to 29 are interpreted as powers of two, meaning 2^12\n"
+          "to 2^29 bytes.\n"
+          "\nThe bidimensional parameter space of LZMA can't be mapped to a linear\n"
           "scale optimal for all files. If your files are large, very repetitive,\n"
-          "etc, you may need to use the --match-length and --dictionary-size\n"
+          "etc, you may need to use the --dictionary-size and --match-length\n"
           "options directly to achieve optimal performance.\n"
           "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
           "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
@@ -181,11 +182,10 @@ static unsigned long long getnum( const char * const ptr,
   if( !errno && tail[0] )
     {
     const int factor = ( tail[1] == 'i' ) ? 1024 : 1000;
-    int exponent = 0, i;
-    bool bad_multiplier = false;
+    int exponent = 0;				/* 0 = bad multiplier */
+    int i;
     switch( tail[0] )
       {
-      case ' ': break;
       case 'Y': exponent = 8; break;
       case 'Z': exponent = 7; break;
       case 'E': exponent = 6; break;
@@ -193,13 +193,10 @@ static unsigned long long getnum( const char * const ptr,
       case 'T': exponent = 4; break;
       case 'G': exponent = 3; break;
       case 'M': exponent = 2; break;
-      case 'K': if( factor == 1024 ) exponent = 1; else bad_multiplier = true;
-                break;
-      case 'k': if( factor == 1000 ) exponent = 1; else bad_multiplier = true;
-                break;
-      default : bad_multiplier = true;
+      case 'K': if( factor == 1024 ) exponent = 1; break;
+      case 'k': if( factor == 1000 ) exponent = 1; break;
       }
-    if( bad_multiplier )
+    if( exponent <= 0 )
       {
       show_error( "Bad multiplier in numerical argument.", 0, true );
       exit( 1 );
@@ -274,7 +271,7 @@ static int open_instream( const char * const name, struct stat * const in_statsp
       const bool can_read = ( i == 0 &&
                               ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
                                 S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
-      const bool no_ofile = to_stdout || program_mode == m_test;
+      const bool no_ofile = ( to_stdout || program_mode == m_test );
       if( i != 0 || ( !S_ISREG( mode ) && ( !can_read || !no_ofile ) ) )
         {
         if( verbosity >= 0 )
@@ -340,13 +337,17 @@ static void set_d_outname( const char * const name, const int i )
   }
 
 
-static bool open_outstream( const bool force )
+static bool open_outstream( const bool force, const bool from_stdin )
   {
+  const mode_t usr_rw = S_IRUSR | S_IWUSR;
+  const mode_t all_rw = usr_rw | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+  const mode_t outfd_mode = from_stdin ? all_rw : usr_rw;
   int flags = O_CREAT | O_WRONLY | O_BINARY;
   if( force ) flags |= O_TRUNC; else flags |= O_EXCL;
 
   outfd = open( output_filename, flags, outfd_mode );
-  if( outfd < 0 && verbosity >= 0 )
+  if( outfd >= 0 ) delete_output_on_interrupt = true;
+  else if( verbosity >= 0 )
     {
     if( errno == EEXIST )
       fprintf( stderr, "%s: Output file '%s' already exists, skipping.\n",
@@ -407,7 +408,11 @@ static void close_and_set_permissions( const struct stat * const in_statsp )
           fchmod( outfd, mode & ~( S_ISUID | S_ISGID | S_ISVTX ) ) != 0 )
         warning = true;
     }
-  if( close( outfd ) != 0 ) cleanup_and_fail( 1 );
+  if( close( outfd ) != 0 )
+    {
+    show_error( "Error closing output file", errno, false );
+    cleanup_and_fail( 1 );
+    }
   outfd = -1;
   delete_output_on_interrupt = false;
   if( in_statsp )
@@ -481,8 +486,8 @@ static int compress( const unsigned long long member_size,
     }
   if( error )
     {
-    show_error( "Not enough memory. Try a smaller dictionary size.", 0, false );
-    cleanup_and_fail( 1 );
+    Pp_show_msg( pp, "Not enough memory. Try a smaller dictionary size." );
+    return 1;
     }
   }
 
@@ -508,8 +513,7 @@ static int compress( const unsigned long long member_size,
           close_and_set_permissions( in_statsp );
           if( !next_filename() )
             { Pp_show_msg( pp, "Too many volume files." ); retval = 1; break; }
-          if( !open_outstream( true ) ) { retval = 1; break; }
-          delete_output_on_interrupt = true;
+          if( !open_outstream( true, !in_statsp ) ) { retval = 1; break; }
           }
         }
       }
@@ -534,8 +538,51 @@ static int compress( const unsigned long long member_size,
   }
 
 
+static unsigned char xdigit( const int value )
+  {
+  if( value >= 0 && value <= 9 ) return '0' + value;
+  if( value >= 10 && value <= 15 ) return 'A' + value - 10;
+  return 0;
+  }
+
+
+static bool show_trailing_data( const uint8_t * const data, const int size,
+                                struct Pretty_print * const pp, const bool all,
+                                const bool ignore_trailing )
+  {
+  if( verbosity >= 4 || !ignore_trailing )
+    {
+    int i;
+    char buf[80];
+    int len = snprintf( buf, sizeof buf, "%strailing data = ",
+                        all ? "" : "first bytes of " );
+    bool text = true;
+    for( i = 0; i < size; ++i )
+      if( !isprint( data[i] ) ) { text = false; break; }
+    if( text )
+      {
+      if( len > 0 && len < (int)sizeof buf )
+      snprintf( buf + len, sizeof buf - len, "'%.*s'", size, (const char *)data );
+      }
+    else
+      {
+      for( i = 0; i < size && len > 0 && len + 3 < (int)sizeof buf; ++i )
+        {
+        if( i > 0 ) buf[len++] = ' ';
+        buf[len++] = xdigit( data[i] >> 4 );
+        buf[len++] = xdigit( data[i] & 0x0F );
+        buf[len] = 0;
+        }
+      }
+    Pp_show_msg( pp, buf );
+    if( !ignore_trailing ) show_error( "Trailing data not allowed.", 0, false );
+    }
+  return ignore_trailing;
+  }
+
+
 static int decompress( const int infd, struct Pretty_print * const pp,
-                       const bool testing )
+                       const bool ignore_trailing, const bool testing )
   {
   unsigned long long partial_file_pos = 0;
   struct Range_decoder rdec;
@@ -549,24 +596,30 @@ static int decompress( const int infd, struct Pretty_print * const pp,
 
   for( first_member = true; ; first_member = false )
     {
-    int result;
+    int result, size;
     unsigned dictionary_size;
     File_header header;
     struct LZ_decoder decoder;
     Rd_reset_member_position( &rdec );
-    Rd_read_data( &rdec, header, Fh_size );
+    size = Rd_read_data( &rdec, header, Fh_size );
     if( Rd_finished( &rdec ) )			/* End Of File */
       {
-      if( first_member )
+      if( first_member || Fh_verify_prefix( header, size ) )
         { Pp_show_msg( pp, "File ends unexpectedly at member header." );
           retval = 2; }
+      else if( size > 0 && !show_trailing_data( header, size, pp,
+                                                true, ignore_trailing ) )
+        retval = 2;
       break;
       }
     if( !Fh_verify_magic( header ) )
       {
-      if( !first_member ) break;		/* trailing garbage */
-      Pp_show_msg( pp, "Bad magic number (file not in lzip format)." );
-      retval = 2; break;
+      if( first_member )
+        { Pp_show_msg( pp, "Bad magic number (file not in lzip format)." );
+          retval = 2; }
+      else if( !show_trailing_data( header, size, pp, false, ignore_trailing ) )
+        retval = 2;
+      break;
       }
     if( !Fh_verify_version( header ) )
       {
@@ -577,8 +630,7 @@ static int decompress( const int infd, struct Pretty_print * const pp,
       retval = 2; break;
       }
     dictionary_size = Fh_get_dictionary_size( header );
-    if( dictionary_size < min_dictionary_size ||
-        dictionary_size > max_dictionary_size )
+    if( !isvalid_ds( dictionary_size ) )
       { Pp_show_msg( pp, "Invalid dictionary size in member header." );
         retval = 2; break; }
 
@@ -586,10 +638,7 @@ static int decompress( const int infd, struct Pretty_print * const pp,
       { Pp_show_msg( pp, 0 ); show_header( dictionary_size ); }
 
     if( !LZd_init( &decoder, &rdec, dictionary_size, outfd ) )
-      {
-      show_error( "Not enough memory.", 0, false );
-      cleanup_and_fail( 1 );
-      }
+      { Pp_show_msg( pp, "Not enough memory." ); retval = 1; break; }
     result = LZd_decode_member( &decoder, pp );
     partial_file_pos += Rd_member_position( &rdec );
     LZd_free( &decoder );
@@ -631,18 +680,16 @@ static void set_signals( void )
 
 void show_error( const char * const msg, const int errcode, const bool help )
   {
-  if( verbosity >= 0 )
+  if( verbosity < 0 ) return;
+  if( msg && msg[0] )
     {
-    if( msg && msg[0] )
-      {
-      fprintf( stderr, "%s: %s", program_name, msg );
-      if( errcode > 0 ) fprintf( stderr, ": %s", strerror( errcode ) );
-      fputc( '\n', stderr );
-      }
-    if( help )
-      fprintf( stderr, "Try '%s --help' for more information.\n",
-               invocation_name );
+    fprintf( stderr, "%s: %s", program_name, msg );
+    if( errcode > 0 ) fprintf( stderr, ": %s", strerror( errcode ) );
+    fputc( '\n', stderr );
     }
+  if( help )
+    fprintf( stderr, "Try '%s --help' for more information.\n",
+             invocation_name );
   }
 
 
@@ -664,18 +711,16 @@ void show_progress( const unsigned long long partial_size,
   static const struct Matchfinder_base * mb = 0;
   static struct Pretty_print * pp = 0;
 
-  if( verbosity >= 2 )
+  if( verbosity < 2 ) return;
+  if( m )					/* initialize static vars */
+    { csize = cfile_size; psize = partial_size; mb = m; pp = p; }
+  if( mb && pp )
     {
-    if( m )					/* initialize static vars */
-      { csize = cfile_size; psize = partial_size; mb = m; pp = p; }
-    if( mb && pp )
-      {
-      const unsigned long long pos = psize + Mb_data_position( mb );
-      if( csize > 0 )
-        fprintf( stderr, "%4llu%%", pos / csize );
-      fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
-      Pp_reset( pp ); Pp_show_msg( pp, 0 );	/* restore cursor position */
-      }
+    const unsigned long long pos = psize + Mb_data_position( mb );
+    if( csize > 0 )
+      fprintf( stderr, "%4llu%%", pos / csize );
+    fprintf( stderr, "  %.1f MB\r", pos / 1000000.0 );
+    Pp_reset( pp ); Pp_show_msg( pp, 0 );	/* restore cursor position */
     }
   }
 
@@ -712,7 +757,9 @@ int main( const int argc, const char * const argv[] )
   int i;
   bool filenames_given = false;
   bool force = false;
+  bool ignore_trailing = true;
   bool keep_input_files = false;
+  bool stdin_used = false;
   bool recompress = false;
   bool to_stdout = false;
   bool zero = false;
@@ -730,6 +777,7 @@ int main( const int argc, const char * const argv[] )
     { '7',  0,                ap_no  },
     { '8',  0,                ap_no  },
     { '9', "best",            ap_no  },
+    { 'a', "trailing-error",  ap_no  },
     { 'b', "member-size",     ap_yes },
     { 'c', "stdout",          ap_no  },
     { 'd', "decompress",      ap_no  },
@@ -769,6 +817,7 @@ int main( const int argc, const char * const argv[] )
       case '5': case '6': case '7': case '8': case '9':
                 zero = ( code == '0' );
                 encoder_options = option_mapping[code-'0']; break;
+      case 'a': ignore_trailing = false; break;
       case 'b': member_size = getnum( arg, 100000, max_member_size ); break;
       case 'c': to_stdout = true; break;
       case 'd': program_mode = m_decompress; break;
@@ -819,7 +868,7 @@ int main( const int argc, const char * const argv[] )
       ( filenames_given || default_output_filename[0] ) )
     set_signals();
 
-  Pp_init( &pp, filenames, num_filenames );
+  Pp_init( &pp, filenames, num_filenames, verbosity );
 
   output_filename = resize_buffer( output_filename, 1 );
   for( i = 0; i < num_filenames; ++i )
@@ -831,6 +880,7 @@ int main( const int argc, const char * const argv[] )
 
     if( !filenames[i][0] || strcmp( filenames[i], "-" ) == 0 )
       {
+      if( stdin_used ) continue; else stdin_used = true;
       input_filename = "";
       infd = STDIN_FILENO;
       if( program_mode != m_test )
@@ -844,11 +894,10 @@ int main( const int argc, const char * const argv[] )
           else
             {
             output_filename = resize_buffer( output_filename,
-                                             strlen( default_output_filename ) + 1 );
+                                strlen( default_output_filename ) + 1 );
             strcpy( output_filename, default_output_filename );
             }
-          outfd_mode = all_rw;
-          if( !open_outstream( force ) )
+          if( !open_outstream( force, true ) )
             {
             if( retval < 1 ) retval = 1;
             close( infd ); infd = -1;
@@ -872,8 +921,7 @@ int main( const int argc, const char * const argv[] )
           if( program_mode == m_compress )
             set_c_outname( input_filename, volume_size > 0 );
           else set_d_outname( input_filename, eindex );
-          outfd_mode = usr_rw;
-          if( !open_outstream( force ) )
+          if( !open_outstream( force, false ) )
             {
             if( retval < 1 ) retval = 1;
             close( infd ); infd = -1;
@@ -883,17 +931,19 @@ int main( const int argc, const char * const argv[] )
         }
       }
 
-    if( !check_tty( infd, program_mode ) ) return 1;
+    if( !check_tty( infd, program_mode ) )
+      {
+      if( retval < 1 ) retval = 1;
+      cleanup_and_fail( retval );
+      }
 
-    if( output_filename[0] && !to_stdout && program_mode != m_test )
-      delete_output_on_interrupt = true;
     in_statsp = input_filename[0] ? &in_stats : 0;
     Pp_set_name( &pp, input_filename );
     if( program_mode == m_compress )
       tmp = compress( member_size, volume_size, infd, &encoder_options, &pp,
                       in_statsp, zero );
     else
-      tmp = decompress( infd, &pp, program_mode == m_test );
+      tmp = decompress( infd, &pp, ignore_trailing, program_mode == m_test );
     if( tmp > retval ) retval = tmp;
     if( tmp && program_mode != m_test ) cleanup_and_fail( retval );
 
